# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    WhatsappMessageTemplateUpdateResponse,
    WhatsappMessageTemplateRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestWhatsappMessageTemplates:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        whatsapp_message_template = client.whatsapp_message_templates.retrieve(
            "id",
        )
        assert_matches_type(WhatsappMessageTemplateRetrieveResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.whatsapp_message_templates.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        whatsapp_message_template = response.parse()
        assert_matches_type(WhatsappMessageTemplateRetrieveResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.whatsapp_message_templates.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            whatsapp_message_template = response.parse()
            assert_matches_type(WhatsappMessageTemplateRetrieveResponse, whatsapp_message_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.whatsapp_message_templates.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        whatsapp_message_template = client.whatsapp_message_templates.update(
            id="id",
        )
        assert_matches_type(WhatsappMessageTemplateUpdateResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        whatsapp_message_template = client.whatsapp_message_templates.update(
            id="id",
            category="MARKETING",
            components=[{}],
        )
        assert_matches_type(WhatsappMessageTemplateUpdateResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.whatsapp_message_templates.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        whatsapp_message_template = response.parse()
        assert_matches_type(WhatsappMessageTemplateUpdateResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.whatsapp_message_templates.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            whatsapp_message_template = response.parse()
            assert_matches_type(WhatsappMessageTemplateUpdateResponse, whatsapp_message_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.whatsapp_message_templates.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        whatsapp_message_template = client.whatsapp_message_templates.delete(
            "id",
        )
        assert whatsapp_message_template is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.whatsapp_message_templates.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        whatsapp_message_template = response.parse()
        assert whatsapp_message_template is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.whatsapp_message_templates.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            whatsapp_message_template = response.parse()
            assert whatsapp_message_template is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.whatsapp_message_templates.with_raw_response.delete(
                "",
            )


class TestAsyncWhatsappMessageTemplates:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        whatsapp_message_template = await async_client.whatsapp_message_templates.retrieve(
            "id",
        )
        assert_matches_type(WhatsappMessageTemplateRetrieveResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.whatsapp_message_templates.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        whatsapp_message_template = await response.parse()
        assert_matches_type(WhatsappMessageTemplateRetrieveResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.whatsapp_message_templates.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            whatsapp_message_template = await response.parse()
            assert_matches_type(WhatsappMessageTemplateRetrieveResponse, whatsapp_message_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.whatsapp_message_templates.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        whatsapp_message_template = await async_client.whatsapp_message_templates.update(
            id="id",
        )
        assert_matches_type(WhatsappMessageTemplateUpdateResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        whatsapp_message_template = await async_client.whatsapp_message_templates.update(
            id="id",
            category="MARKETING",
            components=[{}],
        )
        assert_matches_type(WhatsappMessageTemplateUpdateResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.whatsapp_message_templates.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        whatsapp_message_template = await response.parse()
        assert_matches_type(WhatsappMessageTemplateUpdateResponse, whatsapp_message_template, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.whatsapp_message_templates.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            whatsapp_message_template = await response.parse()
            assert_matches_type(WhatsappMessageTemplateUpdateResponse, whatsapp_message_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.whatsapp_message_templates.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        whatsapp_message_template = await async_client.whatsapp_message_templates.delete(
            "id",
        )
        assert whatsapp_message_template is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.whatsapp_message_templates.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        whatsapp_message_template = await response.parse()
        assert whatsapp_message_template is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.whatsapp_message_templates.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            whatsapp_message_template = await response.parse()
            assert whatsapp_message_template is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.whatsapp_message_templates.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .recording_transcription import RecordingTranscription

__all__ = ["RecordingTranscriptionRetrieveResponse"]


class RecordingTranscriptionRetrieveResponse(BaseModel):
    data: Optional[RecordingTranscription] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["DocumentLinkListResponse"]


class DocumentLinkListResponse(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    document_id: Optional[str] = None
    """Identifies the associated document."""

    linked_record_type: Optional[str] = None
    """The linked resource's record type."""

    linked_resource_id: Optional[str] = None
    """Identifies the linked resource."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["WebhookPortoutFocDateChanged", "Payload"]


class Payload(BaseModel):
    """The webhook payload for the portout.foc_date_changed event"""

    id: Optional[str] = None
    """Identifies the port-out order that have the FOC date changed."""

    foc_date: Optional[datetime] = None
    """ISO 8601 formatted date indicating the new FOC date."""

    user_id: Optional[str] = None
    """Identifies the organization that port-out order belongs to."""


class WebhookPortoutFocDateChanged(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[Literal["portout.status_changed", "portout.foc_date_changed", "portout.new_comment"]] = None
    """Identifies the event type"""

    payload: Optional[Payload] = None
    """The webhook payload for the portout.foc_date_changed event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    portout_id: Optional[str] = None
    """Identifies the port-out order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["CallControlApplicationOutbound"]


class CallControlApplicationOutbound(BaseModel):
    channel_limit: Optional[int] = None
    """
    When set, this will limit the total number of outbound calls to phone numbers
    associated with this connection.
    """

    outbound_voice_profile_id: Optional[str] = None
    """Identifies the associated outbound voice profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.inbound_message_payload import InboundMessagePayload

__all__ = ["InboundMessage"]


class InboundMessage(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["message.received"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    payload: Optional[InboundMessagePayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RimeVoiceSettings"]


class RimeVoiceSettings(TypedDict, total=False):
    type: Required[Literal["rime"]]
    """Voice settings provider type"""

    voice_speed: float
    """Speech speed multiplier. Default is 1.0."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.messaging_10dlc.campaign import usecase_get_cost_params
from ....types.messaging_10dlc.campaign.usecase_get_cost_response import UsecaseGetCostResponse

__all__ = ["UsecaseResource", "AsyncUsecaseResource"]


class UsecaseResource(SyncAPIResource):
    """Campaign operations"""

    @cached_property
    def with_raw_response(self) -> UsecaseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return UsecaseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsecaseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return UsecaseResourceWithStreamingResponse(self)

    def get_cost(
        self,
        *,
        usecase: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsecaseGetCostResponse:
        """
        Get Campaign Cost

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/10dlc/campaign/usecase/cost",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"usecase": usecase}, usecase_get_cost_params.UsecaseGetCostParams),
            ),
            cast_to=UsecaseGetCostResponse,
        )


class AsyncUsecaseResource(AsyncAPIResource):
    """Campaign operations"""

    @cached_property
    def with_raw_response(self) -> AsyncUsecaseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUsecaseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsecaseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncUsecaseResourceWithStreamingResponse(self)

    async def get_cost(
        self,
        *,
        usecase: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UsecaseGetCostResponse:
        """
        Get Campaign Cost

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/10dlc/campaign/usecase/cost",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"usecase": usecase}, usecase_get_cost_params.UsecaseGetCostParams),
            ),
            cast_to=UsecaseGetCostResponse,
        )


class UsecaseResourceWithRawResponse:
    def __init__(self, usecase: UsecaseResource) -> None:
        self._usecase = usecase

        self.get_cost = to_raw_response_wrapper(
            usecase.get_cost,
        )


class AsyncUsecaseResourceWithRawResponse:
    def __init__(self, usecase: AsyncUsecaseResource) -> None:
        self._usecase = usecase

        self.get_cost = async_to_raw_response_wrapper(
            usecase.get_cost,
        )


class UsecaseResourceWithStreamingResponse:
    def __init__(self, usecase: UsecaseResource) -> None:
        self._usecase = usecase

        self.get_cost = to_streamed_response_wrapper(
            usecase.get_cost,
        )


class AsyncUsecaseResourceWithStreamingResponse:
    def __init__(self, usecase: AsyncUsecaseResource) -> None:
        self._usecase = usecase

        self.get_cost = async_to_streamed_response_wrapper(
            usecase.get_cost,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .json import (
    JsonResource,
    AsyncJsonResource,
    JsonResourceWithRawResponse,
    AsyncJsonResourceWithRawResponse,
    JsonResourceWithStreamingResponse,
    AsyncJsonResourceWithStreamingResponse,
)
from .recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)

__all__ = [
    "JsonResource",
    "AsyncJsonResource",
    "JsonResourceWithRawResponse",
    "AsyncJsonResourceWithRawResponse",
    "JsonResourceWithStreamingResponse",
    "AsyncJsonResourceWithStreamingResponse",
    "RecordingsResource",
    "AsyncRecordingsResource",
    "RecordingsResourceWithRawResponse",
    "AsyncRecordingsResourceWithRawResponse",
    "RecordingsResourceWithStreamingResponse",
    "AsyncRecordingsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["BalanceRetrieveResponse", "Data"]


class Data(BaseModel):
    available_credit: Optional[str] = None
    """Available amount to spend (balance + credit limit)"""

    balance: Optional[str] = None
    """The account's current balance."""

    credit_limit: Optional[str] = None
    """The account's credit limit."""

    currency: Optional[str] = None
    """The ISO 4217 currency identifier."""

    pending: Optional[str] = None
    """The account’s pending amount."""

    record_type: Optional[Literal["balance"]] = None
    """Identifies the type of the resource."""


class BalanceRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .comment import Comment
from .._models import BaseModel

__all__ = ["CommentRetrieveResponse", "Data"]


class Data(Comment):
    pass


class CommentRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import integration_secret_list_params, integration_secret_create_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.integration_secret import IntegrationSecret
from ..types.integration_secret_create_response import IntegrationSecretCreateResponse

__all__ = ["IntegrationSecretsResource", "AsyncIntegrationSecretsResource"]


class IntegrationSecretsResource(SyncAPIResource):
    """Store and retrieve integration secrets"""

    @cached_property
    def with_raw_response(self) -> IntegrationSecretsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return IntegrationSecretsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IntegrationSecretsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return IntegrationSecretsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        identifier: str,
        type: Literal["bearer", "basic"],
        token: str | Omit = omit,
        password: str | Omit = omit,
        username: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationSecretCreateResponse:
        """
        Create a new secret with an associated identifier that can be used to securely
        integrate with other services.

        Args:
          identifier: The unique identifier of the secret.

          type: The type of secret.

          token: The token for the secret. Required for bearer type secrets, ignored otherwise.

          password: The password for the secret. Required for basic type secrets, ignored otherwise.

          username: The username for the secret. Required for basic type secrets, ignored otherwise.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/integration_secrets",
            body=maybe_transform(
                {
                    "identifier": identifier,
                    "type": type,
                    "token": token,
                    "password": password,
                    "username": username,
                },
                integration_secret_create_params.IntegrationSecretCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationSecretCreateResponse,
        )

    def list(
        self,
        *,
        filter: integration_secret_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[IntegrationSecret]:
        """
        Retrieve a list of all integration secrets configured by the user.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[type]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/integration_secrets",
            page=SyncDefaultFlatPagination[IntegrationSecret],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    integration_secret_list_params.IntegrationSecretListParams,
                ),
            ),
            model=IntegrationSecret,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an integration secret given its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/integration_secrets/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncIntegrationSecretsResource(AsyncAPIResource):
    """Store and retrieve integration secrets"""

    @cached_property
    def with_raw_response(self) -> AsyncIntegrationSecretsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIntegrationSecretsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIntegrationSecretsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncIntegrationSecretsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        identifier: str,
        type: Literal["bearer", "basic"],
        token: str | Omit = omit,
        password: str | Omit = omit,
        username: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationSecretCreateResponse:
        """
        Create a new secret with an associated identifier that can be used to securely
        integrate with other services.

        Args:
          identifier: The unique identifier of the secret.

          type: The type of secret.

          token: The token for the secret. Required for bearer type secrets, ignored otherwise.

          password: The password for the secret. Required for basic type secrets, ignored otherwise.

          username: The username for the secret. Required for basic type secrets, ignored otherwise.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/integration_secrets",
            body=await async_maybe_transform(
                {
                    "identifier": identifier,
                    "type": type,
                    "token": token,
                    "password": password,
                    "username": username,
                },
                integration_secret_create_params.IntegrationSecretCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationSecretCreateResponse,
        )

    def list(
        self,
        *,
        filter: integration_secret_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[IntegrationSecret, AsyncDefaultFlatPagination[IntegrationSecret]]:
        """
        Retrieve a list of all integration secrets configured by the user.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[type]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/integration_secrets",
            page=AsyncDefaultFlatPagination[IntegrationSecret],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    integration_secret_list_params.IntegrationSecretListParams,
                ),
            ),
            model=IntegrationSecret,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an integration secret given its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/integration_secrets/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class IntegrationSecretsResourceWithRawResponse:
    def __init__(self, integration_secrets: IntegrationSecretsResource) -> None:
        self._integration_secrets = integration_secrets

        self.create = to_raw_response_wrapper(
            integration_secrets.create,
        )
        self.list = to_raw_response_wrapper(
            integration_secrets.list,
        )
        self.delete = to_raw_response_wrapper(
            integration_secrets.delete,
        )


class AsyncIntegrationSecretsResourceWithRawResponse:
    def __init__(self, integration_secrets: AsyncIntegrationSecretsResource) -> None:
        self._integration_secrets = integration_secrets

        self.create = async_to_raw_response_wrapper(
            integration_secrets.create,
        )
        self.list = async_to_raw_response_wrapper(
            integration_secrets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            integration_secrets.delete,
        )


class IntegrationSecretsResourceWithStreamingResponse:
    def __init__(self, integration_secrets: IntegrationSecretsResource) -> None:
        self._integration_secrets = integration_secrets

        self.create = to_streamed_response_wrapper(
            integration_secrets.create,
        )
        self.list = to_streamed_response_wrapper(
            integration_secrets.list,
        )
        self.delete = to_streamed_response_wrapper(
            integration_secrets.delete,
        )


class AsyncIntegrationSecretsResourceWithStreamingResponse:
    def __init__(self, integration_secrets: AsyncIntegrationSecretsResource) -> None:
        self._integration_secrets = integration_secrets

        self.create = async_to_streamed_response_wrapper(
            integration_secrets.create,
        )
        self.list = async_to_streamed_response_wrapper(
            integration_secrets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            integration_secrets.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from .transcription_settings_config_param import TranscriptionSettingsConfigParam

__all__ = ["TranscriptionSettingsParam"]


class TranscriptionSettingsParam(TypedDict, total=False):
    language: str
    """The language of the audio to be transcribed.

    If not set, of if set to `auto`, the model will automatically detect the
    language.
    """

    model: Literal[
        "deepgram/flux",
        "deepgram/nova-3",
        "deepgram/nova-2",
        "azure/fast",
        "distil-whisper/distil-large-v2",
        "openai/whisper-large-v3-turbo",
    ]
    """The speech to text model to be used by the voice assistant.

    All the deepgram models are run on-premise.

    - `deepgram/flux` is optimized for turn-taking but is English-only.
    - `deepgram/nova-3` is multi-lingual with automatic language detection but
      slightly higher latency.
    """

    region: str
    """
    Region on third party cloud providers (currently Azure) if using one of their
    models
    """

    settings: TranscriptionSettingsConfigParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["MissionListEventsParams"]


class MissionListEventsParams(TypedDict, total=False):
    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """Page number (1-based)"""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """Number of items per page"""

    type: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import TYPE_CHECKING
from typing_extensions import Sequence, TypedDict

if TYPE_CHECKING:
    from websockets import Subprotocol
    from websockets.extensions import ClientExtensionFactory


class WebSocketConnectionOptions(TypedDict, total=False):
    """WebSocket connection options copied from `websockets`.

    For example: https://websockets.readthedocs.io/en/stable/reference/asyncio/client.html#websockets.asyncio.client.connect
    """

    extensions: Sequence[ClientExtensionFactory] | None
    """List of supported extensions, in order in which they should be negotiated and run."""

    subprotocols: Sequence[Subprotocol] | None
    """List of supported subprotocols, in order of decreasing preference."""

    compression: str | None
    """The “permessage-deflate” extension is enabled by default. Set compression to None to disable it. See the [compression guide](https://websockets.readthedocs.io/en/stable/topics/compression.html) for details."""

    # limits
    max_size: int | None
    """Maximum size of incoming messages in bytes. None disables the limit."""

    max_queue: int | None | tuple[int | None, int | None]
    """High-water mark of the buffer where frames are received. It defaults to 16 frames. The low-water mark defaults to max_queue // 4. You may pass a (high, low) tuple to set the high-water and low-water marks. If you want to disable flow control entirely, you may set it to None, although that’s a bad idea."""

    write_limit: int | tuple[int, int | None]
    """High-water mark of write buffer in bytes. It is passed to set_write_buffer_limits(). It defaults to 32 KiB. You may pass a (high, low) tuple to set the high-water and low-water marks."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .call_control_command_result_with_conversation_id import CallControlCommandResultWithConversationID

__all__ = ["ActionStartAIAssistantResponse"]


class ActionStartAIAssistantResponse(BaseModel):
    data: Optional[CallControlCommandResultWithConversationID] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["VoicemailPrefResponse"]


class VoicemailPrefResponse(BaseModel):
    enabled: Optional[bool] = None
    """Whether voicemail is enabled."""

    pin: Optional[str] = None
    """The pin used for the voicemail."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.messaging_feature_set import MessagingFeatureSet

__all__ = ["MessagingRetrieveResponse", "Data", "DataFeatures"]


class DataFeatures(BaseModel):
    sms: Optional[MessagingFeatureSet] = None
    """The set of features available for a specific messaging use case (SMS or MMS).

    Features can vary depending on the characteristics the phone number, as well as
    its current product configuration.
    """


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    country_code: Optional[str] = None
    """ISO 3166-1 alpha-2 country code."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    features: Optional[DataFeatures] = None

    messaging_product: Optional[str] = None
    """The messaging product that the number is registered to use"""

    messaging_profile_id: Optional[str] = None
    """Unique identifier for a messaging profile."""

    organization_id: Optional[str] = None
    """The organization that owns this phone number."""

    phone_number: Optional[str] = None
    """+E.164 formatted phone number."""

    record_type: Optional[Literal["messaging_phone_number", "messaging_settings"]] = None
    """Identifies the type of the resource."""

    tags: Optional[List[str]] = None
    """Tags associated with this phone number."""

    traffic_type: Optional[str] = None
    """The messaging traffic or use case for which the number is currently configured."""

    type: Optional[Literal["longcode"]] = None
    """The type of the phone number"""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""


class MessagingRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel
from ..whatsapp_profile_data import WhatsappProfileData

__all__ = ["PhotoUploadResponse"]


class PhotoUploadResponse(BaseModel):
    data: Optional[WhatsappProfileData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .wireless import (
    WirelessResource,
    AsyncWirelessResource,
    WirelessResourceWithRawResponse,
    AsyncWirelessResourceWithRawResponse,
    WirelessResourceWithStreamingResponse,
    AsyncWirelessResourceWithStreamingResponse,
)
from .detail_records_reports import (
    DetailRecordsReportsResource,
    AsyncDetailRecordsReportsResource,
    DetailRecordsReportsResourceWithRawResponse,
    AsyncDetailRecordsReportsResourceWithRawResponse,
    DetailRecordsReportsResourceWithStreamingResponse,
    AsyncDetailRecordsReportsResourceWithStreamingResponse,
)

__all__ = [
    "DetailRecordsReportsResource",
    "AsyncDetailRecordsReportsResource",
    "DetailRecordsReportsResourceWithRawResponse",
    "AsyncDetailRecordsReportsResourceWithRawResponse",
    "DetailRecordsReportsResourceWithStreamingResponse",
    "AsyncDetailRecordsReportsResourceWithStreamingResponse",
    "WirelessResource",
    "AsyncWirelessResource",
    "WirelessResourceWithRawResponse",
    "AsyncWirelessResourceWithRawResponse",
    "WirelessResourceWithStreamingResponse",
    "AsyncWirelessResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel
from .bucket_ids import BucketIDs

__all__ = ["RetrievalTool"]


class RetrievalTool(BaseModel):
    retrieval: BucketIDs

    type: Literal["retrieval"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .profile_update_params import ProfileUpdateParams as ProfileUpdateParams
from .whatsapp_profile_data import WhatsappProfileData as WhatsappProfileData
from .profile_update_response import ProfileUpdateResponse as ProfileUpdateResponse
from .profile_retrieve_response import ProfileRetrieveResponse as ProfileRetrieveResponse
from .calling_setting_update_params import CallingSettingUpdateParams as CallingSettingUpdateParams
from .whatsapp_calling_settings_data import WhatsappCallingSettingsData as WhatsappCallingSettingsData
from .calling_setting_update_response import CallingSettingUpdateResponse as CallingSettingUpdateResponse
from .calling_setting_retrieve_response import CallingSettingRetrieveResponse as CallingSettingRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .waba_settings import WabaSettings

__all__ = ["SettingUpdateResponse"]


class SettingUpdateResponse(BaseModel):
    data: Optional[WabaSettings] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_speak_started import CallSpeakStarted

__all__ = ["CallSpeakStartedWebhookEvent"]


class CallSpeakStartedWebhookEvent(BaseModel):
    data: Optional[CallSpeakStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["OAuthGrantsParams"]


class OAuthGrantsParams(TypedDict, total=False):
    allowed: Required[bool]
    """Whether the grant is allowed"""

    consent_token: Required[str]
    """Consent token"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .loopcount_param import LoopcountParam
from .aws_voice_settings_param import AwsVoiceSettingsParam
from .telnyx_voice_settings_param import TelnyxVoiceSettingsParam
from .eleven_labs_voice_settings_param import ElevenLabsVoiceSettingsParam
from ..shared_params.rime_voice_settings import RimeVoiceSettings
from ..shared_params.azure_voice_settings import AzureVoiceSettings
from ..shared_params.minimax_voice_settings import MinimaxVoiceSettings
from ..shared_params.resemble_voice_settings import ResembleVoiceSettings

__all__ = ["ActionSpeakParams", "VoiceSettings", "VoiceSettingsInworldVoiceSettings"]


class ActionSpeakParams(TypedDict, total=False):
    payload: Required[str]
    """The text or SSML to be converted into speech. There is a 3,000 character limit."""

    voice: Required[str]
    """Specifies the voice used in speech synthesis.

    - Define voices using the format `<Provider>.<Model>.<VoiceId>`. Specifying only
      the provider will give default values for voice_id and model_id.

      **Supported Providers:**

    - **AWS:** Use `AWS.Polly.<VoiceId>` (e.g., `AWS.Polly.Joanna`). For neural
      voices, which provide more realistic, human-like speech, append `-Neural` to
      the `VoiceId` (e.g., `AWS.Polly.Joanna-Neural`). Check the
      [available voices](https://docs.aws.amazon.com/polly/latest/dg/available-voices.html)
      for compatibility.
    - **Azure:** Use `Azure.<VoiceId>` (e.g., `Azure.en-CA-ClaraNeural`,
      `Azure.en-US-BrianMultilingualNeural`,
      `Azure.en-US-Ava:DragonHDLatestNeural`). For a complete list of voices, go to
      [Azure Voice Gallery](https://speech.microsoft.com/portal/voicegallery). Use
      `voice_settings` to configure custom deployments, regions, or API keys.
    - **ElevenLabs:** Use `ElevenLabs.<ModelId>.<VoiceId>` (e.g.,
      `ElevenLabs.eleven_multilingual_v2.21m00Tcm4TlvDq8ikWAM`). The `ModelId` part
      is optional. To use ElevenLabs, you must provide your ElevenLabs API key as an
      integration identifier secret in
      `"voice_settings": {"api_key_ref": "<secret_identifier>"}`. See
      [integration secrets documentation](https://developers.telnyx.com/api/secrets-manager/integration-secrets/create-integration-secret)
      for details. Check
      [available voices](https://elevenlabs.io/docs/api-reference/get-voices).
    - **Telnyx:** Use `Telnyx.<model_id>.<voice_id>` (e.g., `Telnyx.KokoroTTS.af`).
      Use `voice_settings` to configure voice_speed and other synthesis parameters.
    - **Minimax:** Use `Minimax.<ModelId>.<VoiceId>` (e.g.,
      `Minimax.speech-02-hd.Wise_Woman`). Supported models: `speech-02-turbo`,
      `speech-02-hd`, `speech-2.6-turbo`, `speech-2.8-turbo`. Use `voice_settings`
      to configure speed, volume, pitch, and language_boost.
    - **Rime:** Use `Rime.<model_id>.<voice_id>` (e.g., `Rime.Arcana.cove`).
      Supported model_ids: `Arcana`, `Mist`. Use `voice_settings` to configure
      voice_speed.
    - **Resemble:** Use `Resemble.Turbo.<voice_id>` (e.g.,
      `Resemble.Turbo.my_voice`). Only `Turbo` model is supported. Use
      `voice_settings` to configure precision, sample_rate, and format.
    - **Inworld:** Use `Inworld.<ModelId>.<VoiceId>` (e.g., `Inworld.Mini.Loretta`,
      `Inworld.Max.Oliver`). Supported models: `Mini`, `Max`.

    For service_level basic, you may define the gender of the speaker (male or
    female).
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    language: Literal[
        "arb",
        "cmn-CN",
        "cy-GB",
        "da-DK",
        "de-DE",
        "en-AU",
        "en-GB",
        "en-GB-WLS",
        "en-IN",
        "en-US",
        "es-ES",
        "es-MX",
        "es-US",
        "fr-CA",
        "fr-FR",
        "hi-IN",
        "is-IS",
        "it-IT",
        "ja-JP",
        "ko-KR",
        "nb-NO",
        "nl-NL",
        "pl-PL",
        "pt-BR",
        "pt-PT",
        "ro-RO",
        "ru-RU",
        "sv-SE",
        "tr-TR",
    ]
    """The language you want spoken.

    This parameter is ignored when a `Polly.*` voice is specified.
    """

    loop: LoopcountParam
    """The number of times to play the audio file.

    Use `infinity` to loop indefinitely. Defaults to 1.
    """

    payload_type: Literal["text", "ssml"]
    """The type of the provided payload.

    The payload can either be plain text, or Speech Synthesis Markup Language
    (SSML).
    """

    service_level: Literal["basic", "premium"]
    """This parameter impacts speech quality, language options and payload types.

    When using `basic`, only the `en-US` language and payload type `text` are
    allowed.
    """

    stop: str
    """When specified, it stops the current audio being played.

    Specify `current` to stop the current audio being played, and to play the next
    file in the queue. Specify `all` to stop the current audio file being played and
    to also clear all audio files from the queue.
    """

    target_legs: Literal["self", "opposite", "both"]
    """Specifies which legs of the call should receive the spoken audio."""

    voice_settings: VoiceSettings
    """The settings associated with the voice selected"""


class VoiceSettingsInworldVoiceSettings(TypedDict, total=False):
    type: Required[Literal["inworld"]]
    """Voice settings provider type"""


VoiceSettings: TypeAlias = Union[
    ElevenLabsVoiceSettingsParam,
    TelnyxVoiceSettingsParam,
    AwsVoiceSettingsParam,
    MinimaxVoiceSettings,
    AzureVoiceSettings,
    RimeVoiceSettings,
    ResembleVoiceSettings,
    VoiceSettingsInworldVoiceSettings,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.sub_number_order_regulatory_requirement_with_value import SubNumberOrderRegulatoryRequirementWithValue

__all__ = ["NumberOrderPhoneNumber"]


class NumberOrderPhoneNumber(BaseModel):
    id: Optional[str] = None

    bundle_id: Optional[str] = None

    country_code: Optional[str] = None

    deadline: Optional[datetime] = None

    is_block_number: Optional[bool] = None

    locality: Optional[str] = None

    order_request_id: Optional[str] = None

    phone_number: Optional[str] = None

    phone_number_type: Optional[Literal["local", "toll_free", "mobile", "national", "shared_cost", "landline"]] = None

    record_type: Optional[str] = None

    regulatory_requirements: Optional[List[SubNumberOrderRegulatoryRequirementWithValue]] = None

    requirements_met: Optional[bool] = None
    """True if all requirements are met for a phone number, false otherwise."""

    requirements_status: Optional[
        Literal[
            "pending",
            "approved",
            "cancelled",
            "deleted",
            "requirement-info-exception",
            "requirement-info-pending",
            "requirement-info-under-review",
        ]
    ] = None
    """Status of requirements (if applicable)"""

    status: Optional[Literal["pending", "success", "failure"]] = None
    """The status of the phone number in the order."""

    sub_number_order_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_ai_gather_partial_results import CallAIGatherPartialResults

__all__ = ["CallAIGatherPartialResultsWebhookEvent"]


class CallAIGatherPartialResultsWebhookEvent(BaseModel):
    data: Optional[CallAIGatherPartialResults] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["MobileVoiceConnectionCreateParams", "Inbound", "Outbound"]


class MobileVoiceConnectionCreateParams(TypedDict, total=False):
    active: bool

    connection_name: str

    inbound: Inbound

    outbound: Outbound

    tags: SequenceNotStr[str]

    webhook_api_version: Literal["1", "2"]

    webhook_event_failover_url: Optional[str]

    webhook_event_url: Optional[str]

    webhook_timeout_secs: Optional[int]


class Inbound(TypedDict, total=False):
    channel_limit: int


class Outbound(TypedDict, total=False):
    channel_limit: int

    outbound_voice_profile_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.ai.missions.runs import (
    PlanCreateResponse,
    PlanRetrieveResponse,
    PlanUpdateStepResponse,
    PlanAddStepsToPlanResponse,
    PlanGetStepDetailsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPlan:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        plan = client.ai.missions.runs.plan.create(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        )
        assert_matches_type(PlanCreateResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.ai.missions.runs.plan.with_raw_response.create(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanCreateResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.ai.missions.runs.plan.with_streaming_response.create(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanCreateResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.create(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
                steps=[
                    {
                        "description": "description",
                        "sequence": 0,
                        "step_id": "step_id",
                    }
                ],
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.create(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                steps=[
                    {
                        "description": "description",
                        "sequence": 0,
                        "step_id": "step_id",
                    }
                ],
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        plan = client.ai.missions.runs.plan.retrieve(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PlanRetrieveResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.ai.missions.runs.plan.with_raw_response.retrieve(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanRetrieveResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.ai.missions.runs.plan.with_streaming_response.retrieve(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanRetrieveResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.retrieve(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.retrieve(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_add_steps_to_plan(self, client: Telnyx) -> None:
        plan = client.ai.missions.runs.plan.add_steps_to_plan(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        )
        assert_matches_type(PlanAddStepsToPlanResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_add_steps_to_plan(self, client: Telnyx) -> None:
        response = client.ai.missions.runs.plan.with_raw_response.add_steps_to_plan(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanAddStepsToPlanResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_add_steps_to_plan(self, client: Telnyx) -> None:
        with client.ai.missions.runs.plan.with_streaming_response.add_steps_to_plan(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanAddStepsToPlanResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_add_steps_to_plan(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.add_steps_to_plan(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
                steps=[
                    {
                        "description": "description",
                        "sequence": 0,
                        "step_id": "step_id",
                    }
                ],
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.add_steps_to_plan(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                steps=[
                    {
                        "description": "description",
                        "sequence": 0,
                        "step_id": "step_id",
                    }
                ],
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_step_details(self, client: Telnyx) -> None:
        plan = client.ai.missions.runs.plan.get_step_details(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PlanGetStepDetailsResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_step_details(self, client: Telnyx) -> None:
        response = client.ai.missions.runs.plan.with_raw_response.get_step_details(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanGetStepDetailsResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_step_details(self, client: Telnyx) -> None:
        with client.ai.missions.runs.plan.with_streaming_response.get_step_details(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanGetStepDetailsResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_get_step_details(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.get_step_details(
                step_id="step_id",
                mission_id="",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.get_step_details(
                step_id="step_id",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.get_step_details(
                step_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_step(self, client: Telnyx) -> None:
        plan = client.ai.missions.runs.plan.update_step(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PlanUpdateStepResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_step_with_all_params(self, client: Telnyx) -> None:
        plan = client.ai.missions.runs.plan.update_step(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={"foo": "bar"},
            status="pending",
        )
        assert_matches_type(PlanUpdateStepResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update_step(self, client: Telnyx) -> None:
        response = client.ai.missions.runs.plan.with_raw_response.update_step(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = response.parse()
        assert_matches_type(PlanUpdateStepResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update_step(self, client: Telnyx) -> None:
        with client.ai.missions.runs.plan.with_streaming_response.update_step(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = response.parse()
            assert_matches_type(PlanUpdateStepResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update_step(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.update_step(
                step_id="step_id",
                mission_id="",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.update_step(
                step_id="step_id",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            client.ai.missions.runs.plan.with_raw_response.update_step(
                step_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )


class TestAsyncPlan:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        plan = await async_client.ai.missions.runs.plan.create(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        )
        assert_matches_type(PlanCreateResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.missions.runs.plan.with_raw_response.create(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanCreateResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.missions.runs.plan.with_streaming_response.create(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanCreateResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.create(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
                steps=[
                    {
                        "description": "description",
                        "sequence": 0,
                        "step_id": "step_id",
                    }
                ],
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.create(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                steps=[
                    {
                        "description": "description",
                        "sequence": 0,
                        "step_id": "step_id",
                    }
                ],
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        plan = await async_client.ai.missions.runs.plan.retrieve(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PlanRetrieveResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.missions.runs.plan.with_raw_response.retrieve(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanRetrieveResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.missions.runs.plan.with_streaming_response.retrieve(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanRetrieveResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.retrieve(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.retrieve(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_add_steps_to_plan(self, async_client: AsyncTelnyx) -> None:
        plan = await async_client.ai.missions.runs.plan.add_steps_to_plan(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        )
        assert_matches_type(PlanAddStepsToPlanResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_add_steps_to_plan(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.missions.runs.plan.with_raw_response.add_steps_to_plan(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanAddStepsToPlanResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_add_steps_to_plan(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.missions.runs.plan.with_streaming_response.add_steps_to_plan(
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            steps=[
                {
                    "description": "description",
                    "sequence": 0,
                    "step_id": "step_id",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanAddStepsToPlanResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_add_steps_to_plan(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.add_steps_to_plan(
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                mission_id="",
                steps=[
                    {
                        "description": "description",
                        "sequence": 0,
                        "step_id": "step_id",
                    }
                ],
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.add_steps_to_plan(
                run_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                steps=[
                    {
                        "description": "description",
                        "sequence": 0,
                        "step_id": "step_id",
                    }
                ],
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_step_details(self, async_client: AsyncTelnyx) -> None:
        plan = await async_client.ai.missions.runs.plan.get_step_details(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PlanGetStepDetailsResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_step_details(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.missions.runs.plan.with_raw_response.get_step_details(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanGetStepDetailsResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_step_details(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.missions.runs.plan.with_streaming_response.get_step_details(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanGetStepDetailsResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_get_step_details(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.get_step_details(
                step_id="step_id",
                mission_id="",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.get_step_details(
                step_id="step_id",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.get_step_details(
                step_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_step(self, async_client: AsyncTelnyx) -> None:
        plan = await async_client.ai.missions.runs.plan.update_step(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PlanUpdateStepResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_step_with_all_params(self, async_client: AsyncTelnyx) -> None:
        plan = await async_client.ai.missions.runs.plan.update_step(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={"foo": "bar"},
            status="pending",
        )
        assert_matches_type(PlanUpdateStepResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update_step(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.ai.missions.runs.plan.with_raw_response.update_step(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan = await response.parse()
        assert_matches_type(PlanUpdateStepResponse, plan, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update_step(self, async_client: AsyncTelnyx) -> None:
        async with async_client.ai.missions.runs.plan.with_streaming_response.update_step(
            step_id="step_id",
            mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan = await response.parse()
            assert_matches_type(PlanUpdateStepResponse, plan, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update_step(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mission_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.update_step(
                step_id="step_id",
                mission_id="",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `run_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.update_step(
                step_id="step_id",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `step_id` but received ''"):
            await async_client.ai.missions.runs.plan.with_raw_response.update_step(
                step_id="",
                mission_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                run_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import sub_number_orders_report_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.sub_number_orders_report_create_response import SubNumberOrdersReportCreateResponse
from ..types.sub_number_orders_report_retrieve_response import SubNumberOrdersReportRetrieveResponse

__all__ = ["SubNumberOrdersReportResource", "AsyncSubNumberOrdersReportResource"]


class SubNumberOrdersReportResource(SyncAPIResource):
    """Number orders"""

    @cached_property
    def with_raw_response(self) -> SubNumberOrdersReportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SubNumberOrdersReportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubNumberOrdersReportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SubNumberOrdersReportResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        country_code: str | Omit = omit,
        created_at_gt: Union[str, datetime] | Omit = omit,
        created_at_lt: Union[str, datetime] | Omit = omit,
        customer_reference: str | Omit = omit,
        order_request_id: str | Omit = omit,
        status: Literal["pending", "success", "failure"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrdersReportCreateResponse:
        """Create a CSV report for sub number orders.

        The report will be generated
        asynchronously and can be downloaded once complete.

        Args:
          country_code: Filter by country code

          created_at_gt: Filter for orders created after this date

          created_at_lt: Filter for orders created before this date

          customer_reference: Filter by customer reference

          order_request_id: Filter by specific order request ID

          status: Filter by order status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/sub_number_orders_report",
            body=maybe_transform(
                {
                    "country_code": country_code,
                    "created_at_gt": created_at_gt,
                    "created_at_lt": created_at_lt,
                    "customer_reference": customer_reference,
                    "order_request_id": order_request_id,
                    "status": status,
                },
                sub_number_orders_report_create_params.SubNumberOrdersReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrdersReportCreateResponse,
        )

    def retrieve(
        self,
        report_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrdersReportRetrieveResponse:
        """
        Get the status and details of a sub number orders report.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not report_id:
            raise ValueError(f"Expected a non-empty value for `report_id` but received {report_id!r}")
        return self._get(
            f"/sub_number_orders_report/{report_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrdersReportRetrieveResponse,
        )

    def download(
        self,
        report_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Download the CSV file for a completed sub number orders report.

        The report
        status must be 'success' before the file can be downloaded.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not report_id:
            raise ValueError(f"Expected a non-empty value for `report_id` but received {report_id!r}")
        extra_headers = {"Accept": "text/csv", **(extra_headers or {})}
        return self._get(
            f"/sub_number_orders_report/{report_id}/download",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class AsyncSubNumberOrdersReportResource(AsyncAPIResource):
    """Number orders"""

    @cached_property
    def with_raw_response(self) -> AsyncSubNumberOrdersReportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubNumberOrdersReportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubNumberOrdersReportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSubNumberOrdersReportResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        country_code: str | Omit = omit,
        created_at_gt: Union[str, datetime] | Omit = omit,
        created_at_lt: Union[str, datetime] | Omit = omit,
        customer_reference: str | Omit = omit,
        order_request_id: str | Omit = omit,
        status: Literal["pending", "success", "failure"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrdersReportCreateResponse:
        """Create a CSV report for sub number orders.

        The report will be generated
        asynchronously and can be downloaded once complete.

        Args:
          country_code: Filter by country code

          created_at_gt: Filter for orders created after this date

          created_at_lt: Filter for orders created before this date

          customer_reference: Filter by customer reference

          order_request_id: Filter by specific order request ID

          status: Filter by order status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/sub_number_orders_report",
            body=await async_maybe_transform(
                {
                    "country_code": country_code,
                    "created_at_gt": created_at_gt,
                    "created_at_lt": created_at_lt,
                    "customer_reference": customer_reference,
                    "order_request_id": order_request_id,
                    "status": status,
                },
                sub_number_orders_report_create_params.SubNumberOrdersReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrdersReportCreateResponse,
        )

    async def retrieve(
        self,
        report_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubNumberOrdersReportRetrieveResponse:
        """
        Get the status and details of a sub number orders report.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not report_id:
            raise ValueError(f"Expected a non-empty value for `report_id` but received {report_id!r}")
        return await self._get(
            f"/sub_number_orders_report/{report_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubNumberOrdersReportRetrieveResponse,
        )

    async def download(
        self,
        report_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Download the CSV file for a completed sub number orders report.

        The report
        status must be 'success' before the file can be downloaded.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not report_id:
            raise ValueError(f"Expected a non-empty value for `report_id` but received {report_id!r}")
        extra_headers = {"Accept": "text/csv", **(extra_headers or {})}
        return await self._get(
            f"/sub_number_orders_report/{report_id}/download",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=str,
        )


class SubNumberOrdersReportResourceWithRawResponse:
    def __init__(self, sub_number_orders_report: SubNumberOrdersReportResource) -> None:
        self._sub_number_orders_report = sub_number_orders_report

        self.create = to_raw_response_wrapper(
            sub_number_orders_report.create,
        )
        self.retrieve = to_raw_response_wrapper(
            sub_number_orders_report.retrieve,
        )
        self.download = to_raw_response_wrapper(
            sub_number_orders_report.download,
        )


class AsyncSubNumberOrdersReportResourceWithRawResponse:
    def __init__(self, sub_number_orders_report: AsyncSubNumberOrdersReportResource) -> None:
        self._sub_number_orders_report = sub_number_orders_report

        self.create = async_to_raw_response_wrapper(
            sub_number_orders_report.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            sub_number_orders_report.retrieve,
        )
        self.download = async_to_raw_response_wrapper(
            sub_number_orders_report.download,
        )


class SubNumberOrdersReportResourceWithStreamingResponse:
    def __init__(self, sub_number_orders_report: SubNumberOrdersReportResource) -> None:
        self._sub_number_orders_report = sub_number_orders_report

        self.create = to_streamed_response_wrapper(
            sub_number_orders_report.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            sub_number_orders_report.retrieve,
        )
        self.download = to_streamed_response_wrapper(
            sub_number_orders_report.download,
        )


class AsyncSubNumberOrdersReportResourceWithStreamingResponse:
    def __init__(self, sub_number_orders_report: AsyncSubNumberOrdersReportResource) -> None:
        self._sub_number_orders_report = sub_number_orders_report

        self.create = async_to_streamed_response_wrapper(
            sub_number_orders_report.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            sub_number_orders_report.retrieve,
        )
        self.download = async_to_streamed_response_wrapper(
            sub_number_orders_report.download,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["MessagingHostedNumberOrderValidateCodesParams", "VerificationCode"]


class MessagingHostedNumberOrderValidateCodesParams(TypedDict, total=False):
    verification_codes: Required[Iterable[VerificationCode]]


class VerificationCode(TypedDict, total=False):
    code: Required[str]

    phone_number: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .purchase import (
    PurchaseResource,
    AsyncPurchaseResource,
    PurchaseResourceWithRawResponse,
    AsyncPurchaseResourceWithRawResponse,
    PurchaseResourceWithStreamingResponse,
    AsyncPurchaseResourceWithStreamingResponse,
)
from .register import (
    RegisterResource,
    AsyncRegisterResource,
    RegisterResourceWithRawResponse,
    AsyncRegisterResourceWithRawResponse,
    RegisterResourceWithStreamingResponse,
    AsyncRegisterResourceWithStreamingResponse,
)

__all__ = [
    "PurchaseResource",
    "AsyncPurchaseResource",
    "PurchaseResourceWithRawResponse",
    "AsyncPurchaseResourceWithRawResponse",
    "PurchaseResourceWithStreamingResponse",
    "AsyncPurchaseResourceWithStreamingResponse",
    "RegisterResource",
    "AsyncRegisterResource",
    "RegisterResourceWithRawResponse",
    "AsyncRegisterResourceWithRawResponse",
    "RegisterResourceWithStreamingResponse",
    "AsyncRegisterResourceWithStreamingResponse",
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ActionBridgeParams"]


class ActionBridgeParams(TypedDict, total=False):
    call_control_id_to_bridge_with: Required[Annotated[str, PropertyInfo(alias="call_control_id")]]
    """
    The Call Control ID of the call you want to bridge with, can't be used together
    with queue parameter or video_room_id parameter.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    hold_after_unbridge: bool
    """Specifies behavior after the bridge ends.

    If set to `true`, the current leg will be put on hold after unbridge instead of
    being hung up.
    """

    mute_dtmf: Literal["none", "both", "self", "opposite"]
    """When enabled, DTMF tones are not passed to the call participant.

    The webhooks containing the DTMF information will be sent.
    """

    park_after_unbridge: str
    """Specifies behavior after the bridge ends (i.e.

    the opposite leg either hangs up or is transferred). If supplied with the value
    `self`, the current leg will be parked after unbridge. If not set, the default
    behavior is to hang up the leg.
    """

    play_ringtone: bool
    """
    Specifies whether to play a ringtone if the call you want to bridge with has not
    yet been answered.
    """

    prevent_double_bridge: bool
    """
    When set to `true`, it prevents bridging if the target call is already bridged
    to another call. Disabled by default.
    """

    queue: str
    """
    The name of the queue you want to bridge with, can't be used together with
    call_control_id parameter or video_room_id parameter. Bridging with a queue
    means bridging with the first call in the queue. The call will always be removed
    from the queue regardless of whether bridging succeeds. Returns an error when
    the queue is empty.
    """

    record: Literal["record-from-answer"]
    """Start recording automatically after an event. Disabled by default."""

    record_channels: Literal["single", "dual"]
    """
    Defines which channel should be recorded ('single' or 'dual') when `record` is
    specified.
    """

    record_custom_file_name: str
    """The custom recording file name to be used instead of the default `call_leg_id`.

    Telnyx will still add a Unix timestamp suffix.
    """

    record_format: Literal["wav", "mp3"]
    """
    Defines the format of the recording ('wav' or 'mp3') when `record` is specified.
    """

    record_max_length: int
    """
    Defines the maximum length for the recording in seconds when `record` is
    specified. The minimum value is 0. The maximum value is 43200. The default value
    is 0 (infinite).
    """

    record_timeout_secs: int
    """
    The number of seconds that Telnyx will wait for the recording to be stopped if
    silence is detected when `record` is specified. The timer only starts when the
    speech is detected. Please note that call transcription is used to detect
    silence and the related charge will be applied. The minimum value is 0. The
    default value is 0 (infinite).
    """

    record_track: Literal["both", "inbound", "outbound"]
    """The audio track to be recorded.

    Can be either `both`, `inbound` or `outbound`. If only single track is specified
    (`inbound`, `outbound`), `channels` configuration is ignored and it will be
    recorded as mono (single channel).
    """

    record_trim: Literal["trim-silence"]
    """
    When set to `trim-silence`, silence will be removed from the beginning and end
    of the recording.
    """

    ringtone: Literal[
        "at",
        "au",
        "be",
        "bg",
        "br",
        "ch",
        "cl",
        "cn",
        "cz",
        "de",
        "dk",
        "ee",
        "es",
        "fi",
        "fr",
        "gr",
        "hu",
        "il",
        "in",
        "it",
        "jp",
        "lt",
        "mx",
        "my",
        "nl",
        "no",
        "nz",
        "ph",
        "pl",
        "pt",
        "ru",
        "se",
        "sg",
        "th",
        "tw",
        "uk",
        "us-old",
        "us",
        "ve",
        "za",
    ]
    """Specifies which country ringtone to play when `play_ringtone` is set to `true`.

    If not set, the US ringtone will be played.
    """

    video_room_context: str
    """The additional parameter that will be passed to the video conference.

    It is a text field and the user can decide how to use it. For example, you can
    set the participant name or pass JSON text. It can be used only with
    video_room_id parameter.
    """

    video_room_id: str
    """
    The ID of the video room you want to bridge with, can't be used together with
    call_control_id parameter or queue parameter.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Any, List, Type, Generic, Mapping, TypeVar, Optional, cast
from typing_extensions import override

from httpx import Response
from pydantic import Field as FieldInfo

from ._utils import is_mapping
from ._models import BaseModel
from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = [
    "DefaultFlatPaginationMeta",
    "SyncDefaultFlatPagination",
    "AsyncDefaultFlatPagination",
    "SyncDefaultFlatPaginationTopLevelArray",
    "AsyncDefaultFlatPaginationTopLevelArray",
    "DefaultPaginationForLogMessagesMeta",
    "SyncDefaultPaginationForLogMessages",
    "AsyncDefaultPaginationForLogMessages",
    "SyncDefaultPaginationForMessagingTollfree",
    "AsyncDefaultPaginationForMessagingTollfree",
    "SyncDefaultPaginationForQueues",
    "AsyncDefaultPaginationForQueues",
    "DefaultFlatPaginationForInexplicitNumberOrdersMeta",
    "SyncDefaultFlatPaginationForInexplicitNumberOrders",
    "AsyncDefaultFlatPaginationForInexplicitNumberOrders",
    "PerPagePaginationMeta",
    "SyncPerPagePagination",
    "AsyncPerPagePagination",
    "SyncPerPagePaginationV2",
    "AsyncPerPagePaginationV2",
]

_BaseModelT = TypeVar("_BaseModelT", bound=BaseModel)

_T = TypeVar("_T")


class DefaultFlatPaginationMeta(BaseModel):
    page_number: int

    total_pages: int


class SyncDefaultFlatPagination(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    meta: Optional[DefaultFlatPaginationMeta] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = None
        if self.meta is not None:
            if self.meta.page_number is not None:  # pyright: ignore[reportUnnecessaryComparison]
                current_page = self.meta.page_number
        if current_page is None:
            current_page = 1

        total_pages = None
        if self.meta is not None:
            if self.meta.total_pages is not None:  # pyright: ignore[reportUnnecessaryComparison]
                total_pages = self.meta.total_pages
        if total_pages is not None and current_page >= total_pages:
            return None

        return PageInfo(params={"page[number]": current_page + 1})


class AsyncDefaultFlatPagination(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    meta: Optional[DefaultFlatPaginationMeta] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = None
        if self.meta is not None:
            if self.meta.page_number is not None:  # pyright: ignore[reportUnnecessaryComparison]
                current_page = self.meta.page_number
        if current_page is None:
            current_page = 1

        total_pages = None
        if self.meta is not None:
            if self.meta.total_pages is not None:  # pyright: ignore[reportUnnecessaryComparison]
                total_pages = self.meta.total_pages
        if total_pages is not None and current_page >= total_pages:
            return None

        return PageInfo(params={"page[number]": current_page + 1})


class SyncDefaultFlatPaginationTopLevelArray(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        last_page = cast("int | None", self._options.params.get("page[number]")) or 1

        return PageInfo(params={"page[number]": last_page + 1})

    @classmethod
    def build(cls: Type[_BaseModelT], *, response: Response, data: object) -> _BaseModelT:  # noqa: ARG003
        return cls.construct(
            None,
            **{
                **(cast(Mapping[str, Any], data) if is_mapping(data) else {"items": data}),
            },
        )


class AsyncDefaultFlatPaginationTopLevelArray(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        last_page = cast("int | None", self._options.params.get("page[number]")) or 1

        return PageInfo(params={"page[number]": last_page + 1})

    @classmethod
    def build(cls: Type[_BaseModelT], *, response: Response, data: object) -> _BaseModelT:  # noqa: ARG003
        return cls.construct(
            None,
            **{
                **(cast(Mapping[str, Any], data) if is_mapping(data) else {"items": data}),
            },
        )


class DefaultPaginationForLogMessagesMeta(BaseModel):
    page_number: int

    total_pages: int


class SyncDefaultPaginationForLogMessages(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    log_messages: List[_T]
    meta: Optional[DefaultPaginationForLogMessagesMeta] = None

    @override
    def _get_page_items(self) -> List[_T]:
        log_messages = self.log_messages
        if not log_messages:
            return []
        return log_messages

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = None
        if self.meta is not None:
            if self.meta.page_number is not None:  # pyright: ignore[reportUnnecessaryComparison]
                current_page = self.meta.page_number
        if current_page is None:
            current_page = 1

        total_pages = None
        if self.meta is not None:
            if self.meta.total_pages is not None:  # pyright: ignore[reportUnnecessaryComparison]
                total_pages = self.meta.total_pages
        if total_pages is not None and current_page >= total_pages:
            return None

        return PageInfo(params={"page[number]": current_page + 1})


class AsyncDefaultPaginationForLogMessages(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    log_messages: List[_T]
    meta: Optional[DefaultPaginationForLogMessagesMeta] = None

    @override
    def _get_page_items(self) -> List[_T]:
        log_messages = self.log_messages
        if not log_messages:
            return []
        return log_messages

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = None
        if self.meta is not None:
            if self.meta.page_number is not None:  # pyright: ignore[reportUnnecessaryComparison]
                current_page = self.meta.page_number
        if current_page is None:
            current_page = 1

        total_pages = None
        if self.meta is not None:
            if self.meta.total_pages is not None:  # pyright: ignore[reportUnnecessaryComparison]
                total_pages = self.meta.total_pages
        if total_pages is not None and current_page >= total_pages:
            return None

        return PageInfo(params={"page[number]": current_page + 1})


class SyncDefaultPaginationForMessagingTollfree(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    records: List[_T]
    total_records: Optional[int] = None

    @override
    def _get_page_items(self) -> List[_T]:
        records = self.records
        if not records:
            return []
        return records

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        last_page = cast("int | None", self._options.params.get("page")) or 1

        total_pages = self.total_records
        if total_pages is not None and last_page >= total_pages:
            return None

        return PageInfo(params={"page": last_page + 1})


class AsyncDefaultPaginationForMessagingTollfree(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    records: List[_T]
    total_records: Optional[int] = None

    @override
    def _get_page_items(self) -> List[_T]:
        records = self.records
        if not records:
            return []
        return records

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        last_page = cast("int | None", self._options.params.get("page")) or 1

        total_pages = self.total_records
        if total_pages is not None and last_page >= total_pages:
            return None

        return PageInfo(params={"page": last_page + 1})


class SyncDefaultPaginationForQueues(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    queues: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        queues = self.queues
        if not queues:
            return []
        return queues

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        last_page = cast("int | None", self._options.params.get("Page")) or 1

        return PageInfo(params={"Page": last_page + 1})


class AsyncDefaultPaginationForQueues(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    queues: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        queues = self.queues
        if not queues:
            return []
        return queues

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        last_page = cast("int | None", self._options.params.get("Page")) or 1

        return PageInfo(params={"Page": last_page + 1})


class DefaultFlatPaginationForInexplicitNumberOrdersMeta(BaseModel):
    page_number: int

    total_pages: int


class SyncDefaultFlatPaginationForInexplicitNumberOrders(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    meta: Optional[DefaultFlatPaginationForInexplicitNumberOrdersMeta] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = None
        if self.meta is not None:
            if self.meta.page_number is not None:  # pyright: ignore[reportUnnecessaryComparison]
                current_page = self.meta.page_number
        if current_page is None:
            current_page = 1

        total_pages = None
        if self.meta is not None:
            if self.meta.total_pages is not None:  # pyright: ignore[reportUnnecessaryComparison]
                total_pages = self.meta.total_pages
        if total_pages is not None and current_page >= total_pages:
            return None

        return PageInfo(params={"page_number": current_page + 1})


class AsyncDefaultFlatPaginationForInexplicitNumberOrders(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    meta: Optional[DefaultFlatPaginationForInexplicitNumberOrdersMeta] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = None
        if self.meta is not None:
            if self.meta.page_number is not None:  # pyright: ignore[reportUnnecessaryComparison]
                current_page = self.meta.page_number
        if current_page is None:
            current_page = 1

        total_pages = None
        if self.meta is not None:
            if self.meta.total_pages is not None:  # pyright: ignore[reportUnnecessaryComparison]
                total_pages = self.meta.total_pages
        if total_pages is not None and current_page >= total_pages:
            return None

        return PageInfo(params={"page_number": current_page + 1})


class PerPagePaginationMeta(BaseModel):
    page_number: int

    total_pages: int


class SyncPerPagePagination(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    meta: Optional[PerPagePaginationMeta] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = None
        if self.meta is not None:
            if self.meta.page_number is not None:  # pyright: ignore[reportUnnecessaryComparison]
                current_page = self.meta.page_number
        if current_page is None:
            current_page = 1

        total_pages = None
        if self.meta is not None:
            if self.meta.total_pages is not None:  # pyright: ignore[reportUnnecessaryComparison]
                total_pages = self.meta.total_pages
        if total_pages is not None and current_page >= total_pages:
            return None

        return PageInfo(params={"page": current_page + 1})


class AsyncPerPagePagination(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    meta: Optional[PerPagePaginationMeta] = None

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = None
        if self.meta is not None:
            if self.meta.page_number is not None:  # pyright: ignore[reportUnnecessaryComparison]
                current_page = self.meta.page_number
        if current_page is None:
            current_page = 1

        total_pages = None
        if self.meta is not None:
            if self.meta.total_pages is not None:  # pyright: ignore[reportUnnecessaryComparison]
                total_pages = self.meta.total_pages
        if total_pages is not None and current_page >= total_pages:
            return None

        return PageInfo(params={"page": current_page + 1})


class SyncPerPagePaginationV2(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    records: List[_T]
    page: int
    total_records: int = FieldInfo(alias="totalRecords")

    @override
    def _get_page_items(self) -> List[_T]:
        records = self.records
        if not records:
            return []
        return records

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = self.page

        total_pages = self.total_records
        if current_page >= total_pages:
            return None

        return PageInfo(params={"page": current_page + 1})


class AsyncPerPagePaginationV2(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    records: List[_T]
    page: int
    total_records: int = FieldInfo(alias="totalRecords")

    @override
    def _get_page_items(self) -> List[_T]:
        records = self.records
        if not records:
            return []
        return records

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        current_page = self.page

        total_pages = self.total_records
        if current_page >= total_pages:
            return None

        return PageInfo(params={"page": current_page + 1})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["UsageReportGetOptionsResponse", "Data", "DataRecordType"]


class DataRecordType(BaseModel):
    """
    An object following one of the schemas published in https://developers.telnyx.com/docs/api/v2/detail-records
    """

    product_dimensions: Optional[List[str]] = None
    """Telnyx Product Dimensions"""

    product_metrics: Optional[List[str]] = None
    """Telnyx Product Metrics"""

    record_type: Optional[str] = None
    """Telnyx Product type"""


class Data(BaseModel):
    """
    An object following one of the schemas published in https://developers.telnyx.com/docs/api/v2/detail-records
    """

    product: Optional[str] = None
    """Telnyx Product"""

    product_dimensions: Optional[List[str]] = None
    """Telnyx Product Dimensions"""

    product_metrics: Optional[List[str]] = None
    """Telnyx Product Metrics"""

    record_types: Optional[List[DataRecordType]] = None
    """Subproducts if applicable"""


class UsageReportGetOptionsResponse(BaseModel):
    """
    An object following one of the schemas published in https://developers.telnyx.com/docs/api/v2/detail-records
    """

    data: Optional[List[Data]] = None
    """Collection of product description"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_speak_started import ConferenceSpeakStarted

__all__ = ["ConferenceSpeakStartedWebhookEvent"]


class ConferenceSpeakStartedWebhookEvent(BaseModel):
    data: Optional[ConferenceSpeakStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["AzureVoiceSettings"]


class AzureVoiceSettings(TypedDict, total=False):
    type: Required[Literal["azure"]]
    """Voice settings provider type"""

    api_key_ref: str
    """
    The `identifier` for an integration secret that refers to your Azure Speech API
    key.
    """

    deployment_id: str
    """The deployment ID for a custom Azure neural voice."""

    effect: Literal["eq_car", "eq_telecomhp8k"]
    """Audio effect to apply."""

    gender: Literal["Male", "Female"]
    """Voice gender filter."""

    region: str
    """The Azure region for the Speech service (e.g., `eastus`, `westeurope`).

    Required when using a custom API key.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx._utils import parse_datetime
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.reports import (
    MdrUsageReport,
    MdrUsageReportCreateResponse,
    MdrUsageReportDeleteResponse,
    MdrUsageReportRetrieveResponse,
    MdrUsageReportFetchSyncResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMdrUsageReports:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        mdr_usage_report = client.reports.mdr_usage_reports.create(
            aggregation_type="NO_AGGREGATION",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        )
        assert_matches_type(MdrUsageReportCreateResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        mdr_usage_report = client.reports.mdr_usage_reports.create(
            aggregation_type="NO_AGGREGATION",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            profiles="My profile",
        )
        assert_matches_type(MdrUsageReportCreateResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.reports.mdr_usage_reports.with_raw_response.create(
            aggregation_type="NO_AGGREGATION",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = response.parse()
        assert_matches_type(MdrUsageReportCreateResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.reports.mdr_usage_reports.with_streaming_response.create(
            aggregation_type="NO_AGGREGATION",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = response.parse()
            assert_matches_type(MdrUsageReportCreateResponse, mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        mdr_usage_report = client.reports.mdr_usage_reports.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MdrUsageReportRetrieveResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.reports.mdr_usage_reports.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = response.parse()
        assert_matches_type(MdrUsageReportRetrieveResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.reports.mdr_usage_reports.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = response.parse()
            assert_matches_type(MdrUsageReportRetrieveResponse, mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.reports.mdr_usage_reports.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        mdr_usage_report = client.reports.mdr_usage_reports.list()
        assert_matches_type(SyncDefaultFlatPagination[MdrUsageReport], mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        mdr_usage_report = client.reports.mdr_usage_reports.list(
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[MdrUsageReport], mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.reports.mdr_usage_reports.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[MdrUsageReport], mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.reports.mdr_usage_reports.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[MdrUsageReport], mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        mdr_usage_report = client.reports.mdr_usage_reports.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MdrUsageReportDeleteResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.reports.mdr_usage_reports.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = response.parse()
        assert_matches_type(MdrUsageReportDeleteResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.reports.mdr_usage_reports.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = response.parse()
            assert_matches_type(MdrUsageReportDeleteResponse, mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.reports.mdr_usage_reports.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_fetch_sync(self, client: Telnyx) -> None:
        mdr_usage_report = client.reports.mdr_usage_reports.fetch_sync(
            aggregation_type="PROFILE",
        )
        assert_matches_type(MdrUsageReportFetchSyncResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_fetch_sync_with_all_params(self, client: Telnyx) -> None:
        mdr_usage_report = client.reports.mdr_usage_reports.fetch_sync(
            aggregation_type="PROFILE",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            profiles=["My profile"],
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        )
        assert_matches_type(MdrUsageReportFetchSyncResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_fetch_sync(self, client: Telnyx) -> None:
        response = client.reports.mdr_usage_reports.with_raw_response.fetch_sync(
            aggregation_type="PROFILE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = response.parse()
        assert_matches_type(MdrUsageReportFetchSyncResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_fetch_sync(self, client: Telnyx) -> None:
        with client.reports.mdr_usage_reports.with_streaming_response.fetch_sync(
            aggregation_type="PROFILE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = response.parse()
            assert_matches_type(MdrUsageReportFetchSyncResponse, mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncMdrUsageReports:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        mdr_usage_report = await async_client.reports.mdr_usage_reports.create(
            aggregation_type="NO_AGGREGATION",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        )
        assert_matches_type(MdrUsageReportCreateResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        mdr_usage_report = await async_client.reports.mdr_usage_reports.create(
            aggregation_type="NO_AGGREGATION",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            profiles="My profile",
        )
        assert_matches_type(MdrUsageReportCreateResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.reports.mdr_usage_reports.with_raw_response.create(
            aggregation_type="NO_AGGREGATION",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = await response.parse()
        assert_matches_type(MdrUsageReportCreateResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.reports.mdr_usage_reports.with_streaming_response.create(
            aggregation_type="NO_AGGREGATION",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = await response.parse()
            assert_matches_type(MdrUsageReportCreateResponse, mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        mdr_usage_report = await async_client.reports.mdr_usage_reports.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MdrUsageReportRetrieveResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.reports.mdr_usage_reports.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = await response.parse()
        assert_matches_type(MdrUsageReportRetrieveResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.reports.mdr_usage_reports.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = await response.parse()
            assert_matches_type(MdrUsageReportRetrieveResponse, mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.reports.mdr_usage_reports.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        mdr_usage_report = await async_client.reports.mdr_usage_reports.list()
        assert_matches_type(AsyncDefaultFlatPagination[MdrUsageReport], mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        mdr_usage_report = await async_client.reports.mdr_usage_reports.list(
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[MdrUsageReport], mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.reports.mdr_usage_reports.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[MdrUsageReport], mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.reports.mdr_usage_reports.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[MdrUsageReport], mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        mdr_usage_report = await async_client.reports.mdr_usage_reports.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(MdrUsageReportDeleteResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.reports.mdr_usage_reports.with_raw_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = await response.parse()
        assert_matches_type(MdrUsageReportDeleteResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.reports.mdr_usage_reports.with_streaming_response.delete(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = await response.parse()
            assert_matches_type(MdrUsageReportDeleteResponse, mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.reports.mdr_usage_reports.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_fetch_sync(self, async_client: AsyncTelnyx) -> None:
        mdr_usage_report = await async_client.reports.mdr_usage_reports.fetch_sync(
            aggregation_type="PROFILE",
        )
        assert_matches_type(MdrUsageReportFetchSyncResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_fetch_sync_with_all_params(self, async_client: AsyncTelnyx) -> None:
        mdr_usage_report = await async_client.reports.mdr_usage_reports.fetch_sync(
            aggregation_type="PROFILE",
            end_date=parse_datetime("2020-07-01T00:00:00-06:00"),
            profiles=["My profile"],
            start_date=parse_datetime("2020-07-01T00:00:00-06:00"),
        )
        assert_matches_type(MdrUsageReportFetchSyncResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_fetch_sync(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.reports.mdr_usage_reports.with_raw_response.fetch_sync(
            aggregation_type="PROFILE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        mdr_usage_report = await response.parse()
        assert_matches_type(MdrUsageReportFetchSyncResponse, mdr_usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_fetch_sync(self, async_client: AsyncTelnyx) -> None:
        async with async_client.reports.mdr_usage_reports.with_streaming_response.fetch_sync(
            aggregation_type="PROFILE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            mdr_usage_report = await response.parse()
            assert_matches_type(MdrUsageReportFetchSyncResponse, mdr_usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .bucket_ids_param import BucketIDsParam

__all__ = ["RetrievalToolParam"]


class RetrievalToolParam(TypedDict, total=False):
    retrieval: Required[BucketIDsParam]

    type: Required[Literal["retrieval"]]
